/*
  Polling modification to wirirng_analog.c
  Analog input only use existing analogWrite for output
  Modifications Copyright (c) 2014 Forward Computing and Control Pty. Ltd.
  www.forward.com.au
  
  wiring_analog.c - analog input and output
  Part of Arduino - http://www.arduino.cc/

  Copyright (c) 2005-2006 David A. Mellis

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA

  Modified 28 September 2010 by Mark Sproul

  $Id: wiring.c 248 2007-02-03 15:36:30Z mellis $
*/
#include "Arduino.h"
#include "wiring_private.h"
#include "pins_arduino.h"
#include <pollingAnalogRead.h>

static uint8_t polling_analog_reference = DEFAULT;
static boolean analogRefChanged = true; // skip first reading after change in reference
static boolean analogReadingTriggered = false;
static int analogReading = 0; // latest reading.

void pollingAnalogReference(uint8_t mode) {
	// can't actually set the register here because the default setting
	// will connect AVCC and the AREF pin, which would cause a short if
	// there's something connected to AREF.
	if (mode != polling_analog_reference) {
		analogRefChanged = true;
	}	
	polling_analog_reference = mode;
}

bool isAnalogReadingAvailable(void) {
  if (!analogReadingTriggered) {
  	  return false; // not started ADC conversion yet
  }	  
  
  if (bit_is_set(ADCSRA, ADSC)) {
  	  return false; // conversion not finished yet
  }
  // conversion finished
  if (analogRefChanged) {
  	 // skip this one and start another
  	sbi(ADCSRA, ADSC); 	// start the conversion
  	analogRefChanged = false;
	return false;
  } // ELSE
	uint8_t low, high;
	// just return what there is
	// result will not be valid if 
	// isAnalogReadingAvailable() returned FALSE
	
#if defined(ADCSRA) && defined(ADCL)
	// we have to read ADCL first; doing so locks both ADCL
	// and ADCH until ADCH is read.  reading ADCL second would
	// cause the results of each conversion to be discarded,
	// as ADCL and ADCH would be locked when it completed.
	low  = ADCL;
	high = ADCH;
#else
	// we dont have an ADC, return 0
	low  = 0;
	high = 0;
#endif
	// combine the two bytes
	analogReading = (high << 8) | low;
  analogReadingTriggered = false; // can trigger again now
  return true; // reading available
}	
	
void triggerAnalogRead(uint8_t pin) {	
  if (analogReadingTriggered) {
  	  return;
  }	  
  analogReadingTriggered = true;
#if defined(analogPinToChannel)
#if defined(__AVR_ATmega32U4__)
	if (pin >= 18) pin -= 18; // allow for channel or pin numbers
#endif
	pin = analogPinToChannel(pin);
#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
	if (pin >= 54) pin -= 54; // allow for channel or pin numbers
#elif defined(__AVR_ATmega32U4__)
	if (pin >= 18) pin -= 18; // allow for channel or pin numbers
#elif defined(__AVR_ATmega1284__) || defined(__AVR_ATmega1284P__) || defined(__AVR_ATmega644__) || defined(__AVR_ATmega644A__) || defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644PA__)
	if (pin >= 24) pin -= 24; // allow for channel or pin numbers
#else
	if (pin >= 14) pin -= 14; // allow for channel or pin numbers
#endif

#if defined(ADCSRB) && defined(MUX5)
	// the MUX5 bit of ADCSRB selects whether we're reading from channels
	// 0 to 7 (MUX5 low) or 8 to 15 (MUX5 high).
	ADCSRB = (ADCSRB & ~(1 << MUX5)) | (((pin >> 3) & 0x01) << MUX5);
#endif
  
	// set the analog reference (high two bits of ADMUX) and select the
	// channel (low 4 bits).  this also sets ADLAR (left-adjust result)
	// to 0 (the default).
#if defined(ADMUX)
	ADMUX = (polling_analog_reference << 6) | (pin & 0x07);
#endif

	// without a delay, we seem to read from the wrong channel
	//delay(1);

#if defined(ADCSRA) && defined(ADCL)
	// start the conversion
	sbi(ADCSRA, ADSC);
        analogReadingTriggered = true;
//	// ADSC is cleared when the conversion finishes
//	while (bit_is_set(ADCSRA, ADSC));
//
//	// we have to read ADCL first; doing so locks both ADCL
//	// and ADCH until ADCH is read.  reading ADCL second would
//	// cause the results of each conversion to be discarded,
//	// as ADCL and ADCH would be locked when it completed.
//	low  = ADCL;
//	high = ADCH;
#else
	// we dont have an ADC, return 0
//	low  = 0;
//	high = 0;
#endif

//	// combine the two bytes
//	return (high << 8) | low;
}

int getAnalogReading(void) {
	return analogReading;
}
